#ifndef COMPUCELL3DREACTIONADVECTIONDIFFUSIONTAGSSOLVERFE_H
#define COMPUCELL3DREACTIONADVECTIONDIFFUSIONTAGSSOLVERFE_H


#include <CompuCell3D/Steppable.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include "DiffusableVectorContiguous.h"

#include "DiffSecrData.h"

#include <CompuCell3D/Serializer.h>

#include <string>

#include <vector>
#include <set>
#include <map>
#include <iostream>
#include <muParser/muParser.h>

#include "PDESolversDLLSpecifier.h"
#if defined(WIN32) || defined(_WIN32)
        #define WINDOWS true
        #include <windows.h>
#endif
namespace mu{

        class Parser; //mu parser class
};



namespace CompuCell3D {

/**
@author m
*/
//forward declarations
class Potts3D;
class Simulator;
class Cell;
class CellInventory;
class Automaton;


class BoxWatcher;

class DiffusionData;
class SecretionDataFlex;
class ReactionAdvectionDiffusionTagsSolverSerializer;
class TestReactionAdvectionDiffusionTagsSolver; // Testing ReactionAdvectionDiffusionTagsSolverFE
class ParallelUtilsOpenMP;


template <typename Y> class Field3D;
template <typename Y> class Field3DImpl;
template <typename Y> class WatchableField3D;


class ReactionAdvectionDiffusionTagsSolverFE;

class PDESOLVERS_EXPORT SecretionDataRADTags:public SecretionData{
   public:
      typedef void (ReactionAdvectionDiffusionTagsSolverFE::*secrSingleFieldFcnPtr_t)(unsigned int);
      std::vector<secrSingleFieldFcnPtr_t> secretionFcnPtrVec;
};

class PDESOLVERS_EXPORT DiffusionSecretionRADTagsFieldTupple{
   public:
      DiffusionData diffData;
      SecretionDataRADTags secrData;
      DiffusionData * getDiffusionData(){return &diffData;}
      SecretionDataRADTags * getSecretionData(){return &secrData;}
};


class PDESOLVERS_EXPORT ReactionAdvectionDiffusionTagsSolverFE :public DiffusableVectorContiguous<float>
{

  friend class ReactionAdvectionDiffusionTagsSolverSerializer;

  // For Testing
  friend class TestReactionAdvectionDiffusionTagsSolver; // In production version you need to enclose with #ifdef #endif

  public :
   typedef void (ReactionAdvectionDiffusionTagsSolverFE::*diffSecrFcnPtr_t)(void);
   typedef void (ReactionAdvectionDiffusionTagsSolverFE::*secrSingleFieldFcnPtr_t)(unsigned int);
   typedef float precision_t;
   //typedef Array3DBorders<precision_t>::ContainerType Array3D_t;
   typedef Array3DContiguous<precision_t> ConcentrationField_t;

  BoxWatcher *boxWatcherSteppable;

    typedef double (*calcOneDoubles_t)(double); //function pointer with 1 double argument and double return 
    typedef double (*calcTwoDoubles_t)(double,double); //function pointer with 2 double arguments and double return 
    typedef double (*calcThreeDoubles_t)(double,double,double); //function pointer with 2 doubl.e arguments and double return 
    typedef double (*calcFourDoubles_t)(double, double, double, double); //function pointer with 2 doubl.e arguments and double return 
    typedef double (*calcFiveDoubles_t)(double, double, double, double, double); //function pointer with 2 doubl.e arguments and double return 
    typedef double (*calcSixDoubles_t)(double, double, double, double, double, double); //function pointer with 2 doubl.e arguments and double return 
    typedef double (*calcSevenDoubles_t)(double, double, double, double, double, double, double); //function pointer with 2 doubl.e arguments and double return 
    typedef double (*calcEightDoubles_t)(double, double, double, double, double, double, double, double); //function pointer with 2 doubl.e arguments and double return 
    typedef double (*calcNineDoubles_t)(double, double, double, double, double, double, double, double, double); //function pointer with 2 doubl.e arguments and double return 
    typedef double (*calcTenDoubles_t)(double, double, double, double, double, double, double, double, double, double); //function pointer with 2 doubl.e arguments and double return 
    typedef double (*calcElevenDoubles_t)(double, double, double, double, double, double, double, double, double, double, double); //function pointer with 2 doubl.e arguments and double return 
                                
protected:

   Potts3D *potts;
   Simulator *simPtr;
   ParallelUtilsOpenMP *pUtils;

   unsigned int currentStep;
   unsigned int maxDiffusionZ;
   float diffConst;
   float decayConst;
   float deltaX;///spacing
   float deltaT;///time interval
   float dt_dx2; ///ratio delta_t/delta_x^2
   bool UserFuncFlag;
   unsigned int extraTimesPerMCS;
   WatchableField3D<CellG *> *cellFieldG;
   Automaton *automaton;
   
//    std::vector<DiffusionData> diffDataVec;
//    std::vector<SecretionDataFlex> secrDataVec;
   std::vector<bool> periodicBoundaryCheckVector;

   std::vector<BoundaryConditionSpecifier> bcSpecVec;
   std::vector<bool> bcSpecFlagVec;

        bool useBoxWatcher;

        //////std::vector<std::vector<mu::Parser> > parserVec;  
        //////std::vector<vector<double> > variableConcentrationVecMu;
        //////std::vector<double> variableCellTypeMu;

        std::vector<std::vector<mu::Parser> > parserVec;        
        std::vector<vector<double> > variableConcentrationVecMu;
        std::vector<double> variableCellTypeMu;

        std::string cellTypeVariableName;
        
        

   CellInventory *cellInventoryPtr;

   void (ReactionAdvectionDiffusionTagsSolverFE::*diffusePtr)(void);///ptr to member method - Forward Euler diffusion solver
   void (ReactionAdvectionDiffusionTagsSolverFE::*secretePtr)(void);///ptr to member method - Forward Euler diffusion solver
   void diffuse();
   void solveRADTagsEquations();
   void solveRADTagsEquationsSingleField(unsigned int idx);
   void secrete();
   void secreteOnContact();
   void secreteSingleField(unsigned int idx);
   void secreteOnContactSingleField(unsigned int idx);
        void secreteConstantConcentrationSingleField(unsigned int idx);
   void scrarch2Concentration(ConcentrationField_t *concentrationField, ConcentrationField_t *scratchField);
   void outputField( std::ostream & _out,ConcentrationField_t *_concentrationField);
   void readConcentrationField(std::string fileName,ConcentrationField_t *concentrationField);
   void boundaryConditionInit(int idx);
   //void boundaryConditionInit(ConcentrationField_t *concentrationField);
   bool isBoudaryRegion(int x, int y, int z, Dim3D dim);

   unsigned int numberOfFields;
   Dim3D fieldDim;
   Dim3D workFieldDim;
   //float couplingTerm(Point3D & _pt,std::vector<CouplingData> & _couplDataVec,float _currentConcentration);
   void initializeConcentration();

   bool serializeFlag;
   bool readFromFileFlag;
   unsigned int serializeFrequency;

   ReactionAdvectionDiffusionTagsSolverSerializer *serializerPtr;
   bool haveCouplingTerms;
        std::vector<DiffusionSecretionRADTagsFieldTupple>  diffSecrFieldTuppleVec;
        //vector<string> concentrationFieldNameVectorTmp;
   typedef double (ReactionAdvectionDiffusionTagsSolverFE::*fctPtr_t)(int,int);
   double  (ReactionAdvectionDiffusionTagsSolverFE::*fctPtr)(int,int);

    std::vector<std::vector<fctPtr_t> > libPDEVec;
    std::vector<std::vector<std::vector<int> > > fieldDependVec;
    char *dlsym_error;
    void* handle;
        #if defined(WIN32) || defined(_WIN32)
                HINSTANCE handleWIN;
                LPCTSTR path;
                char mod[500];
        #endif

public:

    ReactionAdvectionDiffusionTagsSolverFE();

    virtual ~ReactionAdvectionDiffusionTagsSolverFE();


    virtual void init(Simulator *_simulator, CC3DXMLElement *_xmlData=0);
    virtual void extraInit(Simulator *simulator);

    // Begin Steppable interface
    virtual void start();
    virtual void step(const unsigned int _currentStep);
    virtual void finish();
    
    typedef double (ReactionAdvectionDiffusionTagsSolverFE::*libPDECallptr_t)(int,int);
    libPDECallptr_t libPDECallInit(int NumDeps);

    double libPDECall(int threadNumber,int idx);
    double libPDECall_1(int threadNumber,int idx);
    double libPDECall_2(int threadNumber,int idx);
    double libPDECall_3(int threadNumber,int idx);
    double libPDECall_4(int threadNumber,int idx);
    double libPDECall_5(int threadNumber,int idx);
    double libPDECall_6(int threadNumber,int idx);
    double libPDECall_7(int threadNumber,int idx);
    double libPDECall_8(int threadNumber,int idx);
    double libPDECall_9(int threadNumber,int idx);
    double libPDECall_10(int threadNumber,int idx);
    double libPDECall_11(int threadNumber,int idx);
    double emptyPDECall(int a,int idx);
    // End Steppable interface

    //SteerableObject interface
    virtual void update(CC3DXMLElement *_xmlData, bool _fullInitFlag=false);
    virtual std::string steerableName();
         virtual std::string toString();

};

class PDESOLVERS_EXPORT ReactionAdvectionDiffusionTagsSolverSerializer: public Serializer{
   public:
      ReactionAdvectionDiffusionTagsSolverSerializer():Serializer(){
         solverPtr=0;
         serializedFileExtension="dat";
         currentStep=0;
      }
      ~ReactionAdvectionDiffusionTagsSolverSerializer(){}
      ReactionAdvectionDiffusionTagsSolverFE * solverPtr;
      virtual void serialize();
      virtual void readFromFile();
      void setCurrentStep(unsigned int _currentStep){currentStep=_currentStep;}
   protected:
   unsigned int currentStep;

};





};



#endif
